

import java.io.*;
import java.util.*;
import java.awt.Color;
import java.awt.image.*;
import net.cattaka.mathdrawer.entity.*;
import net.cattaka.mathdrawer.drawer.custom.*;

public class JspfTmp000000 extends CustomPixelShader {
	public JspfTmp000000() {
		StringBuilder descSb = new StringBuilder();
		setName("放射1\n");
descSb.append("放射状に線を描画します。\n");
getCustomMdSettingUtil().addSettingConstraintsInfo("waveType", "Integer", "WaveType", "", "0");
getCustomMdSettingUtil().addSettingConstraintsInfo("startRadius", "Double", "Start Radius", "", "0.0");
getCustomMdSettingUtil().addSettingConstraintsInfo("endRadius", "Double", "End Radius", "", "0.40");
getCustomMdSettingUtil().addSettingConstraintsInfo("width", "Double", "Width", "", "0.01");
getCustomMdSettingUtil().addSettingConstraintsInfo("dumping", "Double", "dumping", "", "0.01");
getCustomMdSettingUtil().addSettingConstraintsInfo("wave", "Double", "Wave", "", "0.05");
getCustomMdSettingUtil().addSettingConstraintsInfo("n", "Integer", "Num", "", "8");
getCustomMdSettingUtil().addSettingConstraintsInfo("scale", "Double", "Scale", "", "2.0");

		this.setDescription(descSb.toString());
	}
	@Override
	public void shadePixel(Color4d c, int x, int y, int w, int h) {
		Integer waveType = Integer.valueOf("0");
waveType = (getInteger("waveType") != null) ? getInteger("waveType") : waveType;
Double startRadius = Double.valueOf("0.0");
startRadius = (getDouble("startRadius") != null) ? getDouble("startRadius") : startRadius;
Double endRadius = Double.valueOf("0.40");
endRadius = (getDouble("endRadius") != null) ? getDouble("endRadius") : endRadius;
Double width = Double.valueOf("0.01");
width = (getDouble("width") != null) ? getDouble("width") : width;
Double dumping = Double.valueOf("0.01");
dumping = (getDouble("dumping") != null) ? getDouble("dumping") : dumping;
Double wave = Double.valueOf("0.05");
wave = (getDouble("wave") != null) ? getDouble("wave") : wave;
Integer n = Integer.valueOf("8");
n = (getInteger("n") != null) ? getInteger("n") : n;
Double scale = Double.valueOf("2.0");
scale = (getDouble("scale") != null) ? getDouble("scale") : scale;

		
class WaveUtil {
	public double calcWave(int waveType, double x) {
		double result;
		switch(waveType) {
			case 0:
				result = cos(x);
				break;
			case 1:
				result = linerWave(x);
				break;
			case 2:
				result = negWave(x);
				break;
			case 3:
				result = sin(x);
				break;
			case 4:
				result = linerWave(x - Math.PI/2.0);
				break;
			case 5:
				result = negWave(x - Math.PI/2.0);
				break;
			default:
				result = 0;
				break;
		}
		return result;
	}

	public double cos(double x) {
		return Math.cos(x);
	}
	public double sin(double x) {
		return Math.sin(x);
	}

	public double linerWave(double x) {
		double div = (Math.PI/2);

		double rad = x % (Math.PI*2);
		if (rad < 0) {
			rad += (Math.PI*2);
		}

		int n = (int)(rad / div);
		double result = rad - div * n;
		result /= Math.PI/2;

		n = n % 4;

		if (n==0) {
			result = 1.0 - result;
		} else if (n==1) {
			result = -result;
		} else if (n==2) {
			result = -(1.0 - result);
		} else if (n==3) {
		}
		return result;
	}

	public double negWave(double x) {
		double div = (Math.PI/2);

		double rad = x % (Math.PI*2);
		if (rad < 0) {
			rad += (Math.PI*2);
		}

		int n = (int)(rad / div);

		n = n % 4;

		double result = 0;
		if (n==0) {
			result = 1.0-Math.sin(rad);
		} else if (n==1) {
			result = Math.sin(rad) - 1.0;
		} else if (n==2) {
			result = -1.0 - Math.sin(rad);
		} else if (n==3) {
			result = Math.sin(rad) + 1.0;
		}
		return result;
	}
}
final WaveUtil waveUtil = new WaveUtil();



class DrawUtil {
	public double drawLine(Vector2d pos, Vector2d p1, Vector2d p2, double width, double dumping) {
		double d;
		int block = MdMath.checkOnBlock(p1,p2,pos);
		if (block == 0) {
			d = Math.abs(MdMath.calcFromLine(pos, p1, p2));
		} else {
			d = Math.min(Vector2d.getDistance(pos,p1),Vector2d.getDistance(pos,p2));
		}

		if (d < width) {
			return 1;
		} else {
			d = d - width;
			if (d<dumping) {
				return (dumping - d) / dumping;
			} else {
				return 0d;
			}
		}
	}
	public double drawFixedWave(Vector2d pos, Vector2d p1, Vector2d p2, double width, double dumping, int waveType, double wave, double scale) {
		return drawFreeWave(pos, p1, p2, width, dumping, waveType, wave, scale / Vector2d.getDistance(p1,p2));
	}
	public double drawFreeWave(Vector2d pos, Vector2d p1, Vector2d p2, double width, double dumping, int waveType, double wave, double scale) {
		double d;

		// 菴咲嶌
		double s;
		{
			Vector2d v1 = new Vector2d(p2);
			Vector2d v2 = new Vector2d(pos);
			v1.sub(p1);
			v2.sub(p1);
			s = Vector2d.getDot(v1,v2) / v1.getLength();
			s = s * scale * Math.PI*2;
		}

		int block = MdMath.checkOnBlock(p1,p2,pos);
		if (block == 0) {
			d = MdMath.calcFromLine(pos, p1, p2);
			d = d - wave * waveUtil.calcWave(waveType, s);
			d = Math.abs(d);
		} else if (block < 0) {
			Vector2d p = new Vector2d(p1);
			Vector2d n = new Vector2d(-(p2.y-p1.y),p2.x-p1.x);
			n.normalize();
			p.scaleAdd(wave * waveUtil.calcWave(waveType, s),n);
			d = Vector2d.getDistance(pos,p);
		} else {
			Vector2d p = new Vector2d(p2);
			Vector2d n = new Vector2d(-(p2.y-p1.y),p2.x-p1.x);
			n.normalize();
			p.scaleAdd(wave * waveUtil.calcWave(waveType, s),n);
			d = Vector2d.getDistance(pos,p);
		}

		if (d < width) {
			return 1;
		} else {
			d = d - width;
			if (d<dumping) {
				return (dumping - d) / dumping;
			} else {
				return 0d;
			}
		}
	}
	public double fillCircle(Vector2d pos, Vector2d p, double radius, double dumping) {
		double d = Vector2d.getDistance(p,pos);
		if (d < radius) {
			return 1;
		} else {
			d = d - radius;
			if (d<dumping) {
				return (dumping - d) / dumping;
			} else {
				return 0d;
			}
		}
	}
	public double drawCircle(Vector2d pos, Vector2d p, double radius, double width, double dumpingInner, double dumpingOuter) {
		double d = Vector2d.getDistance(p,pos);
		if (d < radius) {
			d = radius - d;
			if (d < width) {
				return 1;
			} else if (d < width + dumpingInner) {
				return (dumpingInner - (d - width)) / dumpingInner;
			} else {
				return 0;
			}
		} else {
			d = d - radius;
			if (d < width) {
				return 1;
			} else if (d < width + dumpingOuter) {
				return (dumpingOuter - (d - width)) / dumpingOuter;
			} else {
				return 0;
			}
		}
	}
}
final DrawUtil drawUtil = new DrawUtil();

// 莉･荳逆est
//{
//	Vector2d vec = new Vector2d();
//	Vector2d center = new Vector2d(0.5,0.5);
//	Color4d base = new Color4d();
//	posToVector2d(vec, x, y, w, h);
//	
//	double width = 0.01;
//	double dumping = 0.01;
//	//Vector2d p1 = new Vector2d(0.1,0.1);
//	//Vector2d p2 = new Vector2d(0.9,0.9);
//	Vector2d p1 = new Vector2d(0.1,0.5);
//	Vector2d p2 = new Vector2d(0.9,0.5);
//	
//	baseImage.getColor4d(c,x,y);
//	//c.a = drawUtil.drawLine(vec,p1,p2,width,dumping);
//	//c.a = drawUtil.drawFixedWave(vec,p1,p2,width,dumping, 3, 0.1, 1);
//	//c.a = drawUtil.drawFreeWave(vec,p1,p2,width,dumping, 0, 0.1, 2);
//
//	c.a = drawUtil.drawCircle(vec, new Vector2d(0.25,0.75), 0.2, 0.01, 0.02, 0.05);
////	c.a = drawUtil.fillCircle(vec, new Vector2d(0.75,0.75), 0.2, 0.05);
//}


Vector2d vec = new Vector2d();
Vector2d center = new Vector2d(0.5,0.5);
posToVector2d(vec, x, y, w, h);

Vector2d start = new Vector2d();
Vector2d end = new Vector2d();

double alpha = 0.0;
for (int i=0;i<n;i++) {
	double rad = Math.PI * 2 * (double)i / (double)n;
	start.x = center.x + startRadius * Math.cos(rad);
	start.y = center.y + startRadius * Math.sin(rad);
	end.x = center.x + endRadius * Math.cos(rad);
	end.y = center.y + endRadius * Math.sin(rad);

	alpha = Math.max(alpha, drawUtil.drawFixedWave(vec, start, end, width, dumping, waveType, wave, scale));
}

baseImage.getColor4d(c,x,y);
c.a = alpha;


	}
}
